function NODES = OMNICONT_compress_HSS_dsym_green(C,nbox_max,acc,flag_pot,params)

% The following parameters can be tuned:
  % The relative radius of the proxy circle to the cluster, 1.5 is often good.
  radius_rel = 1.75;
  % The number of points on the proxy circle:
  nproxy = 50;
  if ((flag_pot == 'ht') | (flag_pot == 'h2') | (flag_pot == 'h6'))
    fprintf(1,'WARNING: For Helmholtz problems, "nproxy" must be tuned to the size of the patch.\n')
    fprintf(1,'         Make sure the file has been modified to do so.\n')
  end

% Compute the tree strcture.
NODES  = LOCAL_get_tree(C,nbox_max);
nboxes = size(NODES,2);

% Construct the list of neighbors of any cell.
LIST_NEI = LOCAL_construct_potential_neighborlist(C,NODES,radius_rel);

% Compress all nodes, going from smaller to larger.
for ibox = nboxes:(-1):2

  % Construct the index vectors for ibox.
  if (NODES{5,ibox}==0) % ibox has no sons.
    indskel = NODES{6,ibox} - 1 + (1:NODES{7,ibox});
  elseif (NODES{5,ibox}==1) % ibox has precisely one son
    ison1   = NODES{4,ibox}(1);
    indskel = NODES{20,ison1};
  else % ibox has two sons
    ison1   = NODES{4,ibox}(1);
    ison2   = NODES{4,ibox}(2);
    indskel = [NODES{20,ison1},NODES{20,ison2}];
  end
  
  % Determine a circle that circumscribes the cell:
  [xxc,R] = LOCAL_get_circum_circle(C(:,sort(indskel)));
  Cproxy  = LOCAL_construct_circle(xxc, radius_rel*R, nproxy);
  % Find all contour points inside the circle:
  indskel_maybeinside = [];
  for jbox = LIST_NEI{ibox}
    if (NODES{5,jbox}==0)
      indskel_maybeinside = [indskel_maybeinside,NODES{6,jbox}-1+(1:NODES{7,jbox})];
    else
      jbox_son1 = NODES{4,jbox}(1);
      jbox_son2 = NODES{4,jbox}(2);
      indskel_maybeinside = [indskel_maybeinside,NODES{20,jbox_son1},NODES{20,jbox_son2}];
    end
  end
  relind = find( ((C(1,indskel_maybeinside)-xxc(1)).^2 + ...
                  (C(4,indskel_maybeinside)-xxc(2)).^2) < ((radius_rel*R)^2) );
  indskel_inside = indskel_maybeinside(relind);

  A21proxy = [OMNICONT_construct_A_offd(C,     indskel_inside,        indskel,       flag_pot,params);...
              OMNICONT_construct_A_sing(Cproxy,1:nproxy,       C,     indskel,       flag_pot,params)];
  A12proxy = [OMNICONT_construct_A_offd(C,     indskel,               indskel_inside,flag_pot,params),...
              OMNICONT_construct_A_sing(C,     indskel,        Cproxy,1:nproxy,      flag_pot,params)];
   
  % Compute the skeletons.
  [T,J] = id_decomp([A21proxy;A12proxy'], acc);
  k     = size(T,1);

  % Record the outgoing skeletons:
  NODES{20,ibox} = indskel(J(1:k));
  NODES{30,ibox} = T;
  NODES{31,ibox} = J;
  NODES{ 8,ibox} = length(indskel);
  NODES{ 9,ibox} = k;
  
end

% Construct the matrices representing self-interactions on the leaves.
for ibox = nboxes:(-1):2
  if (NODES{5,ibox}==0)
    ind = NODES{6,ibox} - 1 + (1:NODES{7,ibox});
    NODES{40,ibox} = OMNICONT_construct_A_diag(C,ind,flag_pot,params);
  end
end

% Construct the matrices for sibling interactions.
for ibox = nboxes:(-1):1
  if (NODES{5,ibox}==2)
    ison1 = NODES{4,ibox}(1);
    ison2 = NODES{4,ibox}(2);
    NODES{46,ison1} = OMNICONT_construct_A_offd(C,NODES{20,ison1},NODES{20,ison2},flag_pot,params);
    NODES{46,ison2} = OMNICONT_construct_A_offd(C,NODES{20,ison2},NODES{20,ison1},flag_pot,params);
  end
end

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function NODES = LOCAL_get_tree(C,nbox_max)

ntot = size(C,2);

NODES_0 = cell(10,ntot);

% Construct the top node.
NODES_0{1,1} = [1,ntot];
NODES_0{2,1} = 0;
NODES_0{3,1} = -1;
NODES_0{4,1} = [];
NODES_0{5,1} = 0;
NODES_0{6,1} = 1;
NODES_0{7,1} = ntot;
NODES_0{8,1} = -1; 
NODES_0{9,1} = -1;

% Create smaller nodes via hierarchical subdivision.
% We sweep one level at a time.
ibox_last =  0;
ncreated  =  1;
while (ncreated > 0)
  ibox_first = ibox_last + 1;
  ibox_last  = ibox_last + ncreated;
  ncreated   = 0;
  for ibox = ibox_first:ibox_last
    nbox = NODES_0{7,ibox};
    if (nbox > nbox_max)
      nhalf              = floor(NODES_0{7,ibox}/2);
      if (nhalf > 0)
        ison1            = ibox_last + ncreated + 1;
        NODES_0{4,ibox}  = [NODES_0{4,ibox},ison1];   % Add a child to ibox.
        NODES_0{5,ibox}  = NODES_0{5,ibox}+1;         % Add a child to ibox.
        NODES_0{2,ison1} = NODES_0{2,ibox}+1;         % Set the level           of the child.
        NODES_0{3,ison1} = ibox;                      % Set the parent          of the child.
        NODES_0{4,ison1} = [];                        % Set the child list      of the child.
        NODES_0{5,ison1} = 0;                         % Set the child list      of the child.
        NODES_0{6,ison1} = NODES_0{6,ibox};           % Set the left-most index of the child. 
        NODES_0{7,ison1} = nhalf;                     % Set the number of nodes of the child.
        ncreated         = ncreated + 1;
      end
      if (nhalf < NODES_0{7,ibox})
        ison2            = ibox_last + ncreated + 1;  
        NODES_0{4,ibox}  = [NODES_0{4,ibox},ison2];   % Add a child to ibox.
        NODES_0{5,ibox}  = NODES_0{5,ibox}+1;         % Add a child to ibox. 
        NODES_0{2,ison2} = NODES_0{2,ibox}+1;         % Set the level           of the child.   
        NODES_0{3,ison2} = ibox;                      % Set the parent          of the child.
        NODES_0{4,ison2} = [];                        % Set the child list      of the child. 
        NODES_0{5,ison2} = 0;                         % Set the child list      of the child.   
        NODES_0{6,ison2} = NODES_0{6,ibox} + nhalf;   % Set the left-most index of the child.
        NODES_0{7,ison2} = NODES_0{7,ibox} - nhalf;   % Set the number of nodes of the child. 
        ncreated = ncreated + 1;
      end
    end
  end
end

% This whole section is devoted to the simple task of
% decreasing the size of NODES_0.
% The operation NODES_0 = NODES_0{:,1:nboxes} apparently
% does not work for "cell" structures.
nboxes = ibox_last;
NODES  = cell(50,nboxes);
for ibox = 1:nboxes
  for j = 1:size(NODES_0,1)
    NODES{j,ibox} = NODES_0{j,ibox};
  end
end

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [xxc, R] = LOCAL_get_circum_circle(C);

nloc       = size(C,2);

% First we TRY to create the circle based on the endpoints.
xxc        = 0.5*(C([1,4],1) + C([1,4],end));
distsquare = (C(1,:)-xxc(1)).^2 + (C(4,:)-xxc(2)).^2;
R          = sqrt(max(distsquare));

% The result is an absurdly large circle, then we instead
% base it on the center of mass of the points.
if ( (1.2*R*R) > ( (C(1,1) - C(1,end))^2 + (C(4,1) - C(4,end))^2) )
  xxc        = (1/nloc)*[sum(C(1,:)); sum(C(4,:))];
  distsquare = (C(1,:)-xxc(1)).^2 + (C(4,:)-xxc(2)).^2;
  R          = sqrt(max(distsquare));
end

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function C = LOCAL_construct_circle(xxc, R, n);

tt = linspace(0, 2*pi*(1-1/n), n);
C  = [xxc(1) + R*cos(tt);...
             - R*sin(tt);...
             - R*cos(tt);...
      xxc(2) + R*sin(tt);...
             + R*cos(tt);...
             - R*sin(tt)];

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function LIST_NEI = LOCAL_construct_potential_neighborlist(C,NODES,radius_rel)

nboxes = size(NODES,2);

% Note that in the construction, we temporarily include
% a box in its own list of neighbors.
LIST_NEI    = cell(1,nboxes);
LIST_NEI{1} = [1];
for ibox = 2:nboxes
  [xxc, R] = LOCAL_get_circum_circle(C(:,NODES{6,ibox}-1+(1:NODES{7,ibox})));
  % Collect of list of all the father's neighbors' sons:
  potneis = [];
  for jbox = LIST_NEI{NODES{3,ibox}};
    potneis = [potneis,NODES{4,jbox}];
  end
  % Loop over all potential neighbors to check which ones are actually "close".
  for jbox = potneis
    ind    = NODES{6,jbox} - 1 + (1:NODES{7,jbox});
    distsq = (C(1,ind)-xxc(1)).^2 + (C(4,ind)-xxc(2)).^2;
    if (min(distsq) < radius_rel*radius_rel*R*R)
      LIST_NEI{ibox} = [LIST_NEI{ibox},jbox];
    end
  end
end
% Finally we remove the box itself from the list of neighbors.
for ibox = 2:nboxes
  j = find(LIST_NEI{ibox} == ibox);
  LIST_NEI{ibox} = sort(LIST_NEI{ibox}([1:(j-1),(j+1):end]));
end

return