function A = OMNICONT_construct_A_offd(C,ind1,ind2,flag_pot,params)

ntot     = size(C,2);
curvelen = params(1);
kh       = params(2);
h        = curvelen/ntot;

addweights = true;

[Y_g1,  X_g1  ] = meshgrid(C(1,ind2), C(1,ind1));
[Y_g2,  X_g2  ] = meshgrid(C(4,ind2), C(4,ind1));
[Y_dg1, X_dg1 ] = meshgrid(C(2,ind2), C(2,ind1));
[Y_dg2, X_dg2 ] = meshgrid(C(5,ind2), C(5,ind1));
den = (X_g1 - Y_g1).^2 + (X_g2 - Y_g2).^2;

if (flag_pot == 's2') % This is the single layer pot with 2nd order quadrature.
   den = (X_g1 - Y_g1).^2 + (X_g2 - Y_g2).^2;
   A = -(h/(4*pi))*log(den).*sqrt(Y_dg1.^2 + Y_dg2.^2);
   if (addweights)
      MU2 = [ 0.7518812338640025 + 0.1073866830872157e1; ...
             -0.7225370982867850 - 0.6032109664493744];
      D = abs(ind1'*ones(1,length(ind2)) - ones(length(ind1),1)*ind2);
      D = abs(mod(D + 2, ntot) - 2);
      [ii1, ii2] = find(D.*(D<3));
      for j = 1:length(ii1)
         i1 = ii1(j);
         i2 = ii2(j);
         d  = D(i1,i2);
         A(i1,i2) = (1 + MU2(d))*A(i1,i2);
      end
   end
elseif (flag_pot == 's6') % This is the single layer pot with 6th order quadrature.
   den = (X_g1 - Y_g1).^2 + (X_g2 - Y_g2).^2;
   A = -(h/(4*pi))*log(den).*sqrt(Y_dg1.^2 + Y_dg2.^2);
   if (addweights)
      MU6 = [ 0.2051970990601250e1 + 0.2915391987686505e1;...
             -0.7407035584542865e1 - 0.8797979464048396e1;...
              0.1219590847580216e2 + 0.1365562914252423e2;...
             -0.1064623987147282e2 - 0.1157975479644601e2;...
              0.4799117710681772e1 + 0.5130987287355766e1;...
             -0.8837770983721025   - 0.9342187797694916];
      D = abs(ind1'*ones(1,length(ind2)) - ones(length(ind1),1)*ind2);
      D = abs(mod(D + 6, ntot) - 6);
      [ii1, ii2] = find(D.*(D<7));
      for j = 1:length(ii1)
         i1 = ii1(j);
         i2 = ii2(j);
         d  = D(i1,i2);
         A(i1,i2) = (1 + MU6(d))*A(i1,i2);
      end
   end
elseif (flag_pot == 'dr') % This is the "real" double layer potential.
   xxc = [0;0];
   nn1 = ( Y_dg2./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   nn2 = (-Y_dg1./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   dd  = sqrt((Y_g1 - X_g1).^2 + (Y_g2 - X_g2).^2);
   A   = h*((nn1.*(Y_g1 - X_g1) + nn2.*(Y_g2 - X_g2))./dd).*((-1/(2*pi))*(1./dd)).*sqrt(Y_dg1.^2 + Y_dg2.^2) + ...
         h*(-1/(4*pi))*log((X_g1 - xxc(1)).^2 + (X_g2 - xxc(2)).^2).*sqrt(Y_dg1.^2 + Y_dg2.^2);
elseif (flag_pot == 'ds') % This is the "real" double layer potential + a single layer potential.
   xxc = [0;0];
   nn1 = ( Y_dg2./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   nn2 = (-Y_dg1./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   dd  = sqrt((Y_g1 - X_g1).^2 + (Y_g2 - X_g2).^2);
   A   = h*((nn1.*(Y_g1 - X_g1) + nn2.*(Y_g2 - X_g2))./dd).*((-1/(2*pi))*(1./dd)).*sqrt(Y_dg1.^2 + Y_dg2.^2) + ...
         h*(-1/(2*pi))*log(dd).*sqrt(Y_dg1.^2 + Y_dg2.^2);
   if (addweights)
      MU6 = [ 0.2051970990601250e1 + 0.2915391987686505e1;...
             -0.7407035584542865e1 - 0.8797979464048396e1;...
              0.1219590847580216e2 + 0.1365562914252423e2;...
             -0.1064623987147282e2 - 0.1157975479644601e2;...
              0.4799117710681772e1 + 0.5130987287355766e1;...
             -0.8837770983721025   - 0.9342187797694916];
      D = abs(ind1'*ones(1,length(ind2)) - ones(length(ind1),1)*ind2);
      D = abs(mod(D + 6, ntot) - 6);
      [ii1, ii2] = find(D.*(D<7));
      for j = 1:length(ii1)
         i1 = ii1(j);
         i2 = ii2(j);
         d  = D(i1,i2);
         A(i1,i2) = (1 + MU6(d))*A(i1,i2);
      end
   end
   A = A + h*(-1/(4*pi))*log((X_g1 - xxc(1)).^2 + (X_g2 - xxc(2)).^2).*sqrt(Y_dg1.^2 + Y_dg2.^2);
elseif (flag_pot == 'h2') % This is the double layer Helmholtz potential
   nn1 = ( Y_dg2./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   nn2 = (-Y_dg1./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   dd  = sqrt((Y_g1 - X_g1).^2 + (Y_g2 - X_g2).^2);
   A   = h*(nn1.*(Y_g1 - X_g1) + nn2.*(Y_g2 - X_g2)).*(1./dd).*(-kh*besselh(1, kh*dd)).*sqrt(Y_dg1.^2 + Y_dg2.^2);
   if (addweights)
      MU2 = [ 0.7518812338640025 + 0.1073866830872157e1; ...
             -0.7225370982867850 - 0.6032109664493744];
      D = abs(ind1'*ones(1,length(ind2)) - ones(length(ind1),1)*ind2);
      D = abs(mod(D + 2, ntot) - 2);
      [ii1, ii2] = find(D.*(D<3));
      for j = 1:length(ii1)
         i1 = ii1(j);
         i2 = ii2(j);
         d  = D(i1,i2);
         A(i1,i2) = (1 + MU2(d))*A(i1,i2);
      end
   end
elseif (flag_pot == 'h6') % This is the double layer Helmholtz potential
   nn1 = ( Y_dg2./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   nn2 = (-Y_dg1./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   dd  = sqrt((Y_g1 - X_g1).^2 + (Y_g2 - X_g2).^2);
   A   = h*(nn1.*(Y_g1 - X_g1) + nn2.*(Y_g2 - X_g2)).*(1./dd).*(-kh*besselh(1, kh*dd)).*sqrt(Y_dg1.^2 + Y_dg2.^2);
   if (addweights)
      MU6 = [ 0.2051970990601250e1 + 0.2915391987686505e1;...
             -0.7407035584542865e1 - 0.8797979464048396e1;...
              0.1219590847580216e2 + 0.1365562914252423e2;...
             -0.1064623987147282e2 - 0.1157975479644601e2;...
              0.4799117710681772e1 + 0.5130987287355766e1;...
             -0.8837770983721025   - 0.9342187797694916];
      D = abs(ind1'*ones(1,length(ind2)) - ones(length(ind1),1)*ind2);
      D = abs(mod(D + 6, ntot) - 6);
      [ii1, ii2] = find(D.*(D<7));
      for j = 1:length(ii1)
         i1 = ii1(j);
         i2 = ii2(j);
         d  = D(i1,i2);
         A(i1,i2) = (1 + MU6(d))*A(i1,i2);
      end
   end
elseif (flag_pot == 'ht') % Helmholtz kernel:      [double layer pot] + i kh [single layer pot]
   ima = sqrt(-1);
   nn1 = ( Y_dg2./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   nn2 = (-Y_dg1./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   dd  = sqrt((Y_g1 - X_g1).^2 + (Y_g2 - X_g2).^2);
   A   = ((nn1.*(Y_g1 - X_g1) + nn2.*(Y_g2 - X_g2)).*(1./dd).*(-kh*besselh(1, kh*dd)) + ...
          ima*kh*besselh(0, kh*dd)).*h.*sqrt(Y_dg1.^2 + Y_dg2.^2);
   if (addweights)
      MU6 = [ 0.2051970990601250e1 + 0.2915391987686505e1;...
             -0.7407035584542865e1 - 0.8797979464048396e1;...
              0.1219590847580216e2 + 0.1365562914252423e2;...
             -0.1064623987147282e2 - 0.1157975479644601e2;...
              0.4799117710681772e1 + 0.5130987287355766e1;...
             -0.8837770983721025   - 0.9342187797694916];
      D = abs(ind1'*ones(1,length(ind2)) - ones(length(ind1),1)*ind2);
      D = abs(mod(D + 6, ntot) - 6);
      [ii1, ii2] = find(D.*(D<7));
      for j = 1:length(ii1)
         i1 = ii1(j);
         i2 = ii2(j);
         d  = D(i1,i2);
         A(i1,i2) = (1 + MU6(d))*A(i1,i2);
      end
   end
elseif (flag_pot == 'da') % This is an artificial double layer pot.
   A = h*(1/(2*pi))*(Y_dg2.*(X_g1 - Y_g1) - Y_dg1.*(X_g2 - Y_g2))./den;
elseif (flag_pot == 'sa') % This is an artificial single layer pot.
   A = -(h/(4*pi))*log(den);
elseif (flag_pot == 'sn') % This is an artificial non-symmetric single layer pot.
   A = -(h/(4*pi))*log(den).*sqrt(Y_dg1.^2 + Y_dg2.^2);
end
