function A = OMNICONT_construct_A_sing(C1,ind1,C2,ind2,flag_pot,params)

kh = params(2);
h  = params(3);

[Y_g1,  X_g1  ] = meshgrid(C2(1,ind2), C1(1,ind1));
[Y_g2,  X_g2  ] = meshgrid(C2(4,ind2), C1(4,ind1));
[Y_dg1, X_dg1 ] = meshgrid(C2(2,ind2), C1(2,ind1));
[Y_dg2, X_dg2 ] = meshgrid(C2(5,ind2), C1(5,ind1));
den             = (X_g1 - Y_g1).^2 + (X_g2 - Y_g2).^2;

if (flag_pot == 's2') % This is the single layer pot with 2nd order quadrature.
   den = (X_g1 - Y_g1).^2 + (X_g2 - Y_g2).^2;
   A = -(h/(4*pi))*log(den).*sqrt(Y_dg1.^2 + Y_dg2.^2);
elseif (flag_pot == 's6') % This is the single layer pot with 6th order quadrature.
   den = (X_g1 - Y_g1).^2 + (X_g2 - Y_g2).^2;
   A = -(h/(4*pi))*log(den).*sqrt(Y_dg1.^2 + Y_dg2.^2);
elseif (flag_pot == 'dr') % This is the "real" double layer potential.
   xxc = [0;0];
   nn1 = ( Y_dg2./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   nn2 = (-Y_dg1./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   dd  = sqrt((Y_g1 - X_g1).^2 + (Y_g2 - X_g2).^2);
   A   = h*((nn1.*(Y_g1 - X_g1) + nn2.*(Y_g2 - X_g2))./dd).*((-1/(2*pi))*(1./dd)).*sqrt(Y_dg1.^2 + Y_dg2.^2) + ...
         h*(-1/(4*pi))*log((X_g1 - xxc(1)).^2 + (X_g2 - xxc(2)).^2).*sqrt(Y_dg1.^2 + Y_dg2.^2);
elseif (flag_pot == 'ds') % This is the "real" double layer potential + a single layer potential.
   xxc = [0;0];
   nn1 = ( Y_dg2./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   nn2 = (-Y_dg1./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   dd  = sqrt((Y_g1 - X_g1).^2 + (Y_g2 - X_g2).^2);
   A   = h*((nn1.*(Y_g1 - X_g1) + nn2.*(Y_g2 - X_g2))./dd).*((-1/(2*pi))*(1./dd)).*sqrt(Y_dg1.^2 + Y_dg2.^2) + ...
         h*(-1/(2*pi))*log(dd).*sqrt(Y_dg1.^2 + Y_dg2.^2);
   A   = A + h*(-1/(4*pi))*log((X_g1 - xxc(1)).^2 + (X_g2 - xxc(2)).^2).*sqrt(Y_dg1.^2 + Y_dg2.^2);
elseif (flag_pot == 'h2') % This is the double layer Helmholtz potential
   nn1 = ( Y_dg2./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   nn2 = (-Y_dg1./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   dd  = sqrt((Y_g1 - X_g1).^2 + (Y_g2 - X_g2).^2);
   A   = h*(nn1.*(Y_g1 - X_g1) + nn2.*(Y_g2 - X_g2)).*(1./dd).*(-kh*besselh(1, kh*dd)).*sqrt(Y_dg1.^2 + Y_dg2.^2);
elseif (flag_pot == 'h6') % This is the double layer Helmholtz potential
   nn1 = ( Y_dg2./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   nn2 = (-Y_dg1./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   dd  = sqrt((Y_g1 - X_g1).^2 + (Y_g2 - X_g2).^2);
   A   = h*(nn1.*(Y_g1 - X_g1) + nn2.*(Y_g2 - X_g2)).*(1./dd).*(-kh*besselh(1, kh*dd)).*sqrt(Y_dg1.^2 + Y_dg2.^2);
elseif (flag_pot == 'ht') % Helmholtz kernel:      [double layer pot] + i kh [single layer pot]
   ima = sqrt(-1);
   nn1 = ( Y_dg2./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   nn2 = (-Y_dg1./sqrt(Y_dg1.*Y_dg1 + Y_dg2.*Y_dg2));
   dd  = sqrt((Y_g1 - X_g1).^2 + (Y_g2 - X_g2).^2);
   A   = ((nn1.*(Y_g1 - X_g1) + nn2.*(Y_g2 - X_g2)).*(1./dd).*(-kh*besselh(1, kh*dd)) + ...
          ima*kh*besselh(0, kh*dd)).*h.*sqrt(Y_dg1.^2 + Y_dg2.^2);
elseif (flag_pot == 'da') % This is an artificial double layer pot.
   A = h*(1/(2*pi))*(Y_dg2.*(X_g1 - Y_g1) - Y_dg1.*(X_g2 - Y_g2))./den;
elseif (flag_pot == 'sa') % This is an artificial single layer pot.
   A = -(h/(4*pi))*log(den);
elseif (flag_pot == 'sn') % This is an artificial non-symmetric single layer pot.
   A = -(h/(4*pi))*log(den).*sqrt(Y_dg1.^2 + Y_dg2.^2);
end

return