% This is a test program for the:
%   * HSS format
%   * non-symmetric
% We now compress CONTOURS in two dimensions.

clear
rng('default')
rng(0)

% Set the global parameters.
ntot     = 400;
nbox_max = 50;
acc      = 1e-10;

% Set the geometry:
flag_geom    = 'star';
[C,curvelen] = OMNICONT_construct_contour(ntot,flag_geom);
keyboard

% Set the problem:
flag_pot     = 'dr';
kh           = 1;
params       = [curvelen,kh,curvelen/ntot];
q            = randn(ntot,2);

% Compress using brute force:
if (ntot <= 1000)
  NODES = OMNICONT_compress_HSS_dsym_brute(C,nbox_max,acc,flag_pot,params);
  OMNICONT_draw_tree(NODES)
  OMNI_print_average_ranks(NODES)
  u = OMNI_apply_HSS_dsym(NODES,q);
  A = OMNICONT_construct_A_diag(C,1:ntot,flag_pot,params);
  u_exact = A*q;
  fprintf(1,'Error from brute force compression       = %13.6e\n',norm(u - u_exact)/norm(u_exact))
  u = randn(ntot,1);
  EFG = OMNI_invert_HSS_dsym(NODES);
  q = OMNI_applyinv_HSS_dsym(NODES,EFG,u);
  fprintf(1,'Error ||q - A\\ u||/||q|| = %13.6e (for random q)\n',norm(q - A\u)/norm(q))
end

% Compress using the Green trick force:
tic
NODES = OMNICONT_compress_HSS_dsym_green(C,nbox_max,acc,flag_pot,params);
fprintf('Time required for compressing using the Green trick = %0.3f\n',toc)
%OMNICONT_draw_tree(NODES)
%OMNI_print_average_ranks(NODES)
u = OMNI_apply_HSS_dsym(NODES,q);
if (ntot <= 1000)
  A = OMNICONT_construct_A_diag(C,1:ntot,flag_pot,params);
  u_exact = A*q;
  fprintf(1,'Error from compression using Green-trick = %13.6e\n',norm(u - u_exact)/norm(u_exact))
  EFG = OMNI_invert_HSS_dsym(NODES);
  ACINV = OMNI_applyinv_HSS_dsym(NODES,EFG,eye(ntot));
  fprintf(1,'||inv(A) - ACINV|| = %12.3e\n',norm(inv(A) - ACINV))
else
  n_samp = 20;
  [trash,ind_samp] = sort(rand(1,ntot));
  ind_samp = ind_samp(1:n_samp);
  u_exact = OMNICONT_applyrows(C,flag_pot,params,ind_samp,q);
  fprintf(1,'Error from compression using Green-trick = %13.6e\n',norm(u(ind_samp,:) - u_exact)/norm(u_exact))
end
